/*
 *------------------------------------------------------------------
 *
 * music.h
 * Nov 98
 * 304-427A
 * Sorin Lerner
 *
 * Function prototypes and public data for the music library.
 * You will use this library to generate sounds in the server.
 * Note that the sound generation happens in another process. It is
 * important to call CleanMusic when you are done, so that
 * the sound generating process gets killed. Otherwise sound
 * will be generated long after your server has died.
 *
 * Also, you should make sure that ALL termination paths call
 * CleanMusic. This includes:
 *
 *   - normal termination
 *   - when you exit because of a fatal error.
 *   - when the user kills the server by sending it some signal
 *     like SIGTERM or SIGINT (hint: use signal handlers...)
 *
 *------------------------------------------------------------------
 */    


#ifndef __MUSIC_H__
#define __MUSIC_H__

/*
 * Error constants 
 */

#define OK                         0
#define E_NOT_INITIALIZED       -100
#define E_ALREADY_INITIALIZED   -101
#define E_OUT_OF_MEM            -102
#define E_FREQ_NOT_FOUND        -103
#define E_KILL                  -104
#define E_FORK                  -105
#define E_INVALID_ARG           -106

/*
 * InitMusic(): Initialize the music library. Returns OK on success or
 *   E_OUT_OF_MEM if no more memory is available
 *   E_ALREADY_INITIALIZED if the library has already been initialized
 */
int InitMusic();


/*
 * AddRemoveFreq(): Adds the frequencies in the array Add, and removes
 * those in the array Remove. AddSize specifies the number of entries
 * in the array Add, and RemoveSize specifies the number of entries
 * in the array Remove. RemoveResults is an output array which, on return
 * will contain information as to which frequencies were not found.
 * (See error description below for more details)
 * 
 * Entries can be duplicated. Frequencies are first
 * added, and then removed. Then a process is spawned to continually
 * send data to the audio device, until a new call to AddRemoveFreq is made.
 *
 * Note: Add or Remove can be NULL as long as the associated size is 0.
 * RemoveResults can be NULL if you don't need detailed information
 * as to which frequencies were not found.
 *
 * Returns OK on success or
 *   E_IVALID_ARG if one of Add or Remove is NULL but the associated size is
 *       not 0
 *   E_OUT_OF_MEM if no more memory if availabe
 *   E_KILL if the sound generating process could not be killed
 *   E_FORK if fork failed
 *   E_FREQ_NOT_FOUND if at least one of the frequencies in Remove is not
 *     found. Note that the ones that are found ARE removed.
 *     You will find the ones that were not found by looking at the
 *     RemoveResults array. If RemoveResults[i] == OK then
 *     Remove[i] was found, if RemoveResults[i] == E_FREQ_NOT_FOUND, then
 *     it was not found.
 *
 */

int AddRemoveFreq(unsigned int* Add,unsigned int AddSize,
                  unsigned int* Remove,unsigned int RemoveSize,
                  unsigned int* RemoveResults);

/*
 * CleanMusic(): Cleans the music library. Call this when you are done.
 *
 * Returns OK on success or
 *   E_NOT_INITIALIZED if the library was not initialized
 *   E_KILL if the sound generating process cannot be killed
 */
int CleanMusic();

#endif /* __MUSIC_H__ */
